import requests


class Util:

    api_url = "https://api.murnitur.ai/api"
    endpoint = api_url + "/llm-evaluators/evaluations/save"

    def send_metrics(self, metrics: list, runs: list, headers: any):
        response = requests.post(
            self.endpoint, json={"data": metrics, "metrics": runs}, headers=headers
        )
        return response.status_code == 200 or response.status_code == 201

    def get_preset(self, name: str, api_key: str):
        try:
            response = requests.get(
                url=f"{self.api_url}/presets/sdk?name={name}",
                headers={"x-murnix-trace-token": api_key},
            )
            if response.status_code != 200:
                raise Exception(response.json()["message"])
            return response.status_code, response.json()
        except Exception as e:
            print(e)
            return response.status_code, None



import re
import openlit as lit
from contextlib import contextmanager
from functools import wraps
from enum import Enum
from typing import List, Dict
from openlit.otel.tracing import trace as t
from openlit.semcov import SemanticConvetion
from opentelemetry.trace import SpanKind, Status, StatusCode, Span
from .tracer import setup_tracing
from .util import Util


class Config:
    environment: str
    app_name: str
    api_key: str = ""


class Prompt:
    messages = []
    id: str = ""
    name: str = ""

    def __init__(self, id: str, name: str, messages: List) -> None:
        self.id = id
        self.name = name
        self.messages = messages


class Preset:
    active_version: Prompt = None
    versions: List[Prompt] = []

    def __init__(self, preset):
        active = preset["PresetPrompts"][0]
        self.active_version = Prompt(
            id=active["id"], name=active["name"], messages=active["prompts"]
        )
        self.versions = [
            Prompt(id=curr["id"], name=curr["name"], messages=curr["prompts"])
            for curr in preset["PresetPrompts"]
        ]


class Environment(Enum):
    PRODUCTION = "production"
    DEVELOPMENT = "development"
    STAGING = "staging"


class TracedSpan:
    def __init__(self, span):
        self._span: Span = span

    def set_result(self, result):
        self._span.set_attribute(SemanticConvetion.GEN_AI_CONTENT_COMPLETION, result)

    def set_metadata(self, metadata: Dict):
        self._span.set_attributes(attributes=metadata)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._span.end()


@contextmanager
def tracer(name: str):
    __trace = t.get_tracer_provider()
    with __trace.get_tracer(__name__).start_as_current_span(
        name,
        kind=SpanKind.CLIENT,
    ) as span:
        yield TracedSpan(span)


def trace(wrapped):
    """
    Generates a telemetry wrapper for messages to collect metrics.
    """

    @wraps(wrapped)
    def wrapper(*args, **kwargs):
        __trace = t.get_tracer_provider()
        with __trace.get_tracer(__name__).start_as_current_span(
            name=wrapped.__name__,
            kind=SpanKind.CLIENT,
        ) as span:
            try:
                response = wrapped(*args, **kwargs)
                span.set_attribute(
                    SemanticConvetion.GEN_AI_CONTENT_COMPLETION, response
                )
                span.set_status(Status(StatusCode.OK))
            except Exception as e:
                response = None
                span.record_exception(e)
                span.set_status(Status(StatusCode.ERROR))
                lit.logging.error(f"Error in {wrapped.__name__}: {e}", exc_info=True)

            # Adding function arguments as metadata
            try:
                span.set_attribute("function.args", str(args))
                span.set_attribute("function.kwargs", str(kwargs))
                span.set_attribute(
                    SemanticConvetion.GEN_AI_APPLICATION_NAME, Config.app_name
                )
                span.set_attribute(
                    SemanticConvetion.GEN_AI_ENVIRONMENT, Config.environment
                )
            except Exception as meta_exception:
                lit.logging.error(
                    f"Failed to set metadata for {wrapped.__name__}: {meta_exception}",
                    exc_info=True,
                )

            return response

    return wrapper


def set_api_key(api_key: str):
    Config.api_key = api_key


def get_api_key():
    return Config.api_key


def load_preset(name):
    code, content = Util().get_preset(name=name, api_key=Config.api_key)
    if content:
        return Preset(content)
    return None


def format_prompt(messages: List, params: Dict):
    def replace_match(match):
        variable_name = match.group(1)
        return str(params.get(variable_name, f"{{{{{variable_name}}}}}"))

    replaced_templates = []
    for template in messages:
        new_template = {}
        for key, value in template.items():
            if isinstance(value, str):
                new_value = re.sub(r"\{\{([\w-]+)\}\}", replace_match, value)
                new_template[key] = new_value
            else:
                new_template[key] = value
        replaced_templates.append(new_template)

    return replaced_templates


def init(project_name: str, environment: Environment = Environment.DEVELOPMENT):
    if len(Config.api_key.strip()) == 0:
        print("Please provide a valid API key!")
        return

    Config.app_name = project_name
    Config.environment = environment.value

    # Setup tracer
    tracer = setup_tracing(
        application_name=project_name,
        environment=environment.value,
        otlp_headers=f"x-murnix-trace-token={Config.api_key}",
        disable_batch=False,
        tracer=None,
    )

    lit.init(
        environment=environment.value,
        application_name=project_name,
        disable_metrics=True,
        tracer=tracer,
    )



import os
from ..util import Util
from typing import List, Dict, Optional
from deepeval import evaluate
from deepeval.test_case import LLMTestCase
from deepeval.dataset import EvaluationDataset
from deepeval.metrics import (
    AnswerRelevancyMetric,
    HallucinationMetric,
    BiasMetric,
    FaithfulnessMetric,
    ContextualRelevancyMetric,
    ContextualPrecisionMetric,
    ToxicityMetric,
    SummarizationMetric,
)
from .pii import PIIMetric


class Evaluation:

    def __init__(
        self, openai_api_key: str, murnix_api_key: str, model: str = "gpt-3.5-turbo"
    ):
        os.environ["OPENAI_API_KEY"] = openai_api_key
        self.model = model
        self.api_key = murnix_api_key

        pii_metric = PIIMetric(threshold=0.1)
        hallucination_metric = HallucinationMetric(
            threshold=0.3, model=model, strict_mode=True
        )
        answer_relevancy_metric = AnswerRelevancyMetric(
            threshold=0.5, model=model, strict_mode=True
        )
        faithfullness_metric = FaithfulnessMetric(model=model, strict_mode=True)
        bias_metric = BiasMetric(model=model, strict_mode=True)
        context_relevancy_metric = ContextualRelevancyMetric(
            model=model, strict_mode=True
        )
        context_precision_metric = ContextualPrecisionMetric(
            model=model, strict_mode=True
        )
        toxicity_metric = ToxicityMetric(model=model, strict_mode=True)
        summarization_metric = SummarizationMetric(model=model, strict_mode=True)

        self.all_metrics = {
            "hallucination": hallucination_metric,
            "faithfulness": faithfullness_metric,
            "relevancy": answer_relevancy_metric,
            "bias": bias_metric,
            "context-relevancy": context_relevancy_metric,
            "context-precision": context_precision_metric,
            "toxicity": toxicity_metric,
            "summarization": summarization_metric,
            "pii": pii_metric,
        }

    def run_suite(
        self,
        metrics: List[str],
        dataset: List[Dict],
        save_output: Optional[bool] = True,
        async_mode: Optional[bool] = True,
    ) -> List[Dict]:
        try:
            eval_dataset = EvaluationDataset(
                test_cases=[
                    LLMTestCase(
                        input=c["input"],
                        actual_output=c["output"],
                        context=c["context"],
                        retrieval_context=c["retrieval_context"],
                        expected_output=c["ground_truth"],
                    )
                    for c in dataset
                ]
            )
            result = self.__evaluate_outputs__(
                dataset=eval_dataset, metrics=metrics, async_mode=async_mode
            )
            if save_output:
                Util().send_metrics(
                    metrics=result,
                    runs=metrics,
                    headers={"x-murnix-trace-token": self.api_key},
                )
            return result
        except Exception as e:
            print(f"Error occured: {e}")

    def __evaluate_outputs__(
        self, dataset: EvaluationDataset, metrics: list[str], async_mode: bool
    ):
        allowed_metrics = []
        for m in metrics:
            if m in self.all_metrics:
                allowed_metrics.append(self.all_metrics[m])
        result = evaluate(
            dataset,
            allowed_metrics,
            write_cache=False,
            use_cache=False,
            print_results=False,
            run_async=async_mode,
            show_indicator=False,
        )
        output = []
        for r in result:
            metrics = [
                {
                    "name": m.__name__,
                    "cost": m.evaluation_cost,
                    "model": m.evaluation_model,
                    "threshold": m.threshold,
                    "score": m.score,
                    "reason": m.reason,
                    "error": m.error,
                    "success": m.success,
                }
                for m in r.metrics
            ]

            output.append(
                {
                    "input": r.input,
                    "success": r.success,
                    "output": r.actual_output,
                    "ground_truth": r.expected_output,
                    "context": r.context[0] if r.context is not None else "",
                    "retrieval_context": (
                        r.retrieval_context[0]
                        if r.retrieval_context is not None
                        else ""
                    ),
                    "metrics": metrics,
                }
            )

        return output



from setuptools import setup, find_packages

with open("README.md", encoding="utf-8") as readme_file:
    long_description = readme_file.read()


setup(
    name="murnitur",
    version="0.1",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Caleb Okpara",
    author_email="support@murnitur.com",
    packages=find_packages(),
    install_requires=[
        "requests",
        "openlit",
        "deepeval",
        "transformers>=4.39.3",
        "pandas",
    ],
    python_requires=">=3.9",
)
